from django.contrib import admin
from django.utils.translation import gettext_lazy as _

from . import models
from .forms import UserCreationForm


@admin.register(models.User)
class UserAdmin(admin.ModelAdmin):
    list_display = ('username', 'email', 'first_name', 'last_name', 'is_staff')
    list_filter = ('is_staff', 'is_superuser', 'is_active')
    search_fields = ('username', 'email', 'first_name', 'last_name')
    ordering = ('username',)

    fieldsets = (
        (None, {'fields': ('username', 'email', 'password')}),
        (_('Personal Info'), {'fields': ('first_name', 'last_name')}),
        (_('Permissions'), {'fields': ('is_active', 'is_staff', 'is_admin', 'is_superuser')}),
        (_('Important Dates'), {'fields': ('last_login', 'date_joined')}),
    )
    add_fieldsets = (
        (
            None,
            {
                "classes": ("wide",),
                "fields": ("username", "password", "re_password"),
            },
        ),
    )

    add_form = UserCreationForm



from django.urls import path
from .views import GetCSRFToken, IsAuthenticated, LoginView, LogoutView

urlpatterns = [
    path('csrf-cookie/', GetCSRFToken.as_view()),
    path('is-authenticated/', IsAuthenticated.as_view()),
    path('login/', LoginView.as_view(), name='login'),
    path('logout/', LogoutView.as_view(), name='logout'),
]



from django.contrib.auth import login, logout
from django.views.generic import CreateView
from django.views.decorators.csrf import csrf_protect, ensure_csrf_cookie
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.decorators import method_decorator

from rest_framework.views import APIView
from rest_framework.viewsets import ModelViewSet
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.status import (
    HTTP_200_OK,
    HTTP_201_CREATED,
    HTTP_204_NO_CONTENT,
    HTTP_400_BAD_REQUEST,
    HTTP_401_UNAUTHORIZED,
    HTTP_409_CONFLICT,
    HTTP_500_INTERNAL_SERVER_ERROR,
)

from .models import User
from .forms import UserCreationForm
from .serializers import UserSerializer


class RegisterView(CreateView):
    model = User
    form_class = UserCreationForm

    def form_valid(self, form):
        login(self.request, form.save())
        return redirect(self.request.GET.get('next', reverse(self.request.META.get('HTTP_REFERER') if self.request.META.get('HTTP_REFERER') else 'home')))


@method_decorator(ensure_csrf_cookie, name='dispatch')
class GetCSRFToken(APIView):
    permission_classes = (AllowAny, )

    def get(self, request, format=None):
        return Response({'success': 'CSRF Token cookie set'}, status=HTTP_200_OK)


class IsAuthenticated(APIView):
    permission_classes = (AllowAny, )

    def get(self, request, format=None):
        user = request.user
        if user.is_authenticated:
            return Response({'isAuthenticated': 'Yes'}, status=HTTP_200_OK)
        return Response({'isAuthenticated': 'No'}, status=HTTP_200_OK)


@method_decorator(csrf_protect, name='dispatch')
class LoginView(APIView):
    permission_classes = (AllowAny,)

    def post(self, request, format=None):
        try:
            email = request.data.get('email')
            password = request.data.get('password')
            if not email:
                return Response({'email_error': 'you must enter the email'}, status=HTTP_401_UNAUTHORIZED)
            if not password:
                return Response({'password_error': 'you must enter the password'}, status=HTTP_401_UNAUTHORIZED)
            user = User.objects.filter(email=email)
            if user.exists():
                user = user[0]
                if user.check_password(password):
                    login(request, user)
                    return Response({'success': 'login successfully'}, status=HTTP_200_OK)
                return Response({'password_error': "password isn't correct"}, status=HTTP_401_UNAUTHORIZED)
            return Response({'email_error': "email doesn't exist"}, status=HTTP_401_UNAUTHORIZED)
        except Exception as e:
            return Response({'error': str(e)}, status=HTTP_500_INTERNAL_SERVER_ERROR)


class LogoutView(APIView):

    def post(self, request, format=None):
        try:
            logout(request)
            return Response({}, status=HTTP_204_NO_CONTENT)
        except:
            return Response({'error': 'something went wrong when loging out'}, status=HTTP_500_INTERNAL_SERVER_ERROR)


class UserViewSet(ModelViewSet):
    queryset = User.objects.all()
    serializer_class = UserSerializer
    permission_classes = (AllowAny,)

    def create(self, request, *args, **kwargs):
        try:
            email = request.data.get('email')
            username = request.data.get('username')
            password = request.data.get('password')
            re_password = request.data.get('re_password')
            if not email:
                return Response({'email_error': 'you must enter the email'}, status=HTTP_400_BAD_REQUEST)
            if not username:
                return Response({'username_error': 'you must enter the username'}, status=HTTP_400_BAD_REQUEST)
            if not password:
                return Response({'password_error': 'you must enter the password'}, status=HTTP_400_BAD_REQUEST)
            if not re_password:
                return Response({'password_error': 'you must enter the password confirmation'}, status=HTTP_400_BAD_REQUEST)
            if password != re_password:
                return Response({'password_error': 'Passwords do not match'}, status=HTTP_409_CONFLICT)
            if User.objects.filter(email=email).exists():
                return Response({'email_error': 'Email already exists'}, status=HTTP_409_CONFLICT)
            if User.objects.filter(username=username).exists():
                return Response({'username_error': 'Username already exists'}, status=HTTP_409_CONFLICT)
            if len(password) < 8:
                return Response({'password_error': 'Password must be more than 8 characters'}, status=HTTP_400_BAD_REQUEST)
            serializer = self.get_serializer(data=self.request.data)
            if serializer.is_valid():
                self.perform_create(serializer)
                return Response({'success': 'User created successfully'}, status=HTTP_201_CREATED)
            return Response(serializer.errors, status=HTTP_400_BAD_REQUEST)
        except Exception as e:
            return Response({'error': str(e)}, status=HTTP_500_INTERNAL_SERVER_ERROR)



from django.forms import EmailField, CharField, EmailInput, TextInput, PasswordInput
from django.contrib.auth.forms import UserCreationForm as BaseUserCreationForm
from django.utils.translation import gettext_lazy as _

from .models import User


class UserCreationForm(BaseUserCreationForm):
    email = EmailField(
        label=_("Email Address"),
        widget=EmailInput(attrs={'class': 'input-field', 'placeholder': _('Enter your email'), 'required': True}),
    )
    username = CharField(
        label=_("Username"),
        widget=TextInput(attrs={'class': 'input-field', 'placeholder': _('Choose a username'), 'required': True}),
    )
    password = CharField(
        label=_("Password"),
        widget=PasswordInput(attrs={'class': 'input-field', 'placeholder': _('Enter your password'), 'required': True}),
    )
    re_password = CharField(
        label=_("Confirm Password"),
        widget=PasswordInput(attrs={'class': 'input-field', 'placeholder': _('Confirm your password'), 'required': True}),
    )

    class Meta:
        model = User
        fields = ['email', 'username', 'password', 're_password']



from rest_framework.serializers import ModelSerializer
from .models import User


class UserSerializer(ModelSerializer):
    class Meta:
        model = User
        fields = ('id', 'username', 'email', 'date_joined', 'last_login', 'is_active', 'is_staff', 'is_superuser')

    def create(self, validated_data):
        password = validated_data.pop('password', None)
        instance = self.Meta.model(**validated_data)
        if password is not None:
            instance.set_password(password)
        instance.save()
        return instance



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

extras_require = {
    "rest_framework": ["djangorestframework"],
    "filters": ["django-filter"],
}

setup(
    name="django-email-accounts",
    version="0.5.0",
    author="Fathi Abdelmalek",
    author_email="abdelmalek.fathi.2001@gmail.com",
    description="A Django app for user management with email-based authentication.",
    license = "OSI Approved :: GNU General Public License v3 (GPLv3)",
long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/fathiabdelmalek/django-email-accounts",
    packages=['email_accounts'],
    package_data={
        "email_accounts": ["templates/**/*", "static/**/*"],
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Internet",
        "Topic :: Software Development",
    ],
    python_requires=">=3.6",
    install_requires=[
        "Django",
    ],
    extras_require=extras_require,
)



from django.db import models
from django.contrib.auth.models import BaseUserManager, AbstractBaseUser, PermissionsMixin
from django.utils.translation import gettext_lazy as _


class UserManager(BaseUserManager):
    def create_user(self, email, username, password=None, **extra_fields):
        if not email:
            raise ValueError("You must enter email address")
        if not username:
            raise ValueError("You must enter username")
        email = self.normalize_email(email)
        user = self.model(email=email, username=username, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, email, username, password, **extra_fields):
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)
        if extra_fields.get("is_staff") is not True:
            raise ValueError("Superuser must have is_staff=True.")
        if extra_fields.get("is_superuser") is not True:
            raise ValueError("Superuser must have is_superuser=True.")
        return self.create_user(email, username, password, **extra_fields)


class User(AbstractBaseUser, PermissionsMixin):
    email = models.EmailField(_("email address"), unique=True)
    username = models.CharField(
        _("username"),
        max_length=150,
        unique=True,
        help_text=_('Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.'),
        error_messages={"unique": _("A user with that username already exists.")},
    )
    first_name = models.CharField(_("first name"), max_length=150, blank=True)
    last_name = models.CharField(_("last name"), max_length=150, blank=True)
    date_joined = models.DateTimeField(verbose_name=_("join date"), auto_now_add=True)
    last_login = models.DateTimeField(verbose_name=_("last login"), auto_now=True)
    is_active = models.BooleanField(_("active"), default=True)
    is_staff = models.BooleanField(_("staff status"), default=False)

    objects = UserManager()

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = ['username']

    class Meta:
        verbose_name = _("user")
        verbose_name_plural = _("users")

    def __str__(self):
        return self.username

    def get_full_name(self):
        return f"{self.first_name} {self.last_name}"

    def get_short_name(self):
        return self.first_name



from django.urls import path
from django.contrib.auth.views import (
    LoginView,
    LogoutView,
    PasswordChangeView,
    PasswordChangeDoneView,
    PasswordResetView,
    PasswordResetDoneView,
    PasswordResetConfirmView,
    PasswordResetCompleteView,
)
from .views import RegisterView
from .utils import anonymous_required

urlpatterns = [
    path('register/',
         anonymous_required(RegisterView.as_view(template_name='email_accounts/register.html')),
         name='register'),
    path('login/',
         anonymous_required(LoginView.as_view(template_name='email_accounts/login.html')),
         name='login'),
    path('logout/',
         LogoutView.as_view(),
         name='logout'),
    path('password_change/',
         PasswordChangeView.as_view(template_name='email_accounts/password/change/1.html'),
         name='password_change'),
    path('password_change/done',
         PasswordChangeDoneView.as_view(template_name='email_accounts/password/change/2.html'),
         name='password_change_done'),
    path('password_reset/',
         PasswordResetView.as_view(template_name='email_accounts/password/reset/1.html'),
         name='password_reset'),
    path('password_reset/done/',
         PasswordResetDoneView.as_view(template_name='email_accounts/password/reset/2.html'),
         name='password_reset_done'),
    path('password_reset/confirm/<uidb64>/<token>/',
         PasswordResetConfirmView.as_view(template_name='email_accounts/password/reset/3.html'),
         name='password_reset_confirm'),
    path('password_reset/complete/',
         PasswordResetCompleteView.as_view(template_name='email_accounts/password/reset/4.html'),
         name='password_reset_complete'),
]



import django_filters
from .models import User


class UserFilter(django_filters.FilterSet):
    class Meta:
        model = User
        fields = {
            'username': ['iexact', 'icontains'],
            'email': ['iexact', 'icontains'],
            'is_staff': ['exact'],
            'is_superuser': ['exact'],
        }
